@@ -1,3 +1,3 @@
 /** Private support classes for server-sent events. */
-@javax.annotation.ParametersAreNonnullByDefault
+@okhttp3.internal.annotations.EverythingIsNonNull
 package okhttp3.internal.sse;
