@@ -16,6 +16,8 @@
 package okhttp3;
 
 import java.io.IOException;
+import java.net.ServerSocket;
+import java.net.Socket;
 import java.net.SocketTimeoutException;
 import java.util.Arrays;
 import java.util.Locale;
@@ -62,7 +64,7 @@ public final class InterceptorTest {
         .url("https://localhost:1/")
         .build();
 
-    final Response interceptorResponse = new Response.Builder()
+    Response interceptorResponse = new Response.Builder()
         .request(request)
         .protocol(Protocol.HTTP_1_1)
         .code(200)
@@ -71,11 +73,8 @@ public final class InterceptorTest {
         .build();
 
     client = client.newBuilder()
-        .addInterceptor(new Interceptor() {
-          @Override public Response intercept(Chain chain) throws IOException {
-            return interceptorResponse;
-          }
-        }).build();
+        .addInterceptor(chain -> interceptorResponse)
+        .build();
 
     Response response = client.newCall(request).execute();
     assertSame(interceptorResponse, response);
@@ -84,17 +83,13 @@ public final class InterceptorTest {
   @Test public void networkInterceptorsCannotShortCircuitResponses() throws Exception {
     server.enqueue(new MockResponse().setResponseCode(500));
 
-    Interceptor interceptor = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        return new Response.Builder()
-            .request(chain.request())
-            .protocol(Protocol.HTTP_1_1)
-            .code(200)
-            .message("Intercepted!")
-            .body(ResponseBody.create(MediaType.get("text/plain; charset=utf-8"), "abc"))
-            .build();
-      }
-    };
+    Interceptor interceptor = chain -> new Response.Builder()
+        .request(chain.request())
+        .protocol(Protocol.HTTP_1_1)
+        .code(200)
+        .message("Intercepted!")
+        .body(ResponseBody.create(MediaType.get("text/plain; charset=utf-8"), "abc"))
+        .build();
     client = client.newBuilder()
         .addNetworkInterceptor(interceptor)
         .build();
@@ -116,11 +111,9 @@ public final class InterceptorTest {
     server.enqueue(new MockResponse());
     server.enqueue(new MockResponse());
 
-    Interceptor interceptor = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        chain.proceed(chain.request());
-        return chain.proceed(chain.request());
-      }
+    Interceptor interceptor = chain -> {
+      chain.proceed(chain.request());
+      return chain.proceed(chain.request());
     };
     client = client.newBuilder()
         .addNetworkInterceptor(interceptor)
@@ -142,15 +135,13 @@ public final class InterceptorTest {
   @Test public void networkInterceptorsCannotChangeServerAddress() throws Exception {
     server.enqueue(new MockResponse().setResponseCode(500));
 
-    Interceptor interceptor = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        Address address = chain.connection().route().address();
-        String sameHost = address.url().host();
-        int differentPort = address.url().port() + 1;
-        return chain.proceed(chain.request().newBuilder()
-            .url("http://" + sameHost + ":" + differentPort + "/")
-            .build());
-      }
+    Interceptor interceptor = chain -> {
+      Address address = chain.connection().route().address();
+      String sameHost = address.url().host();
+      int differentPort = address.url().port() + 1;
+      return chain.proceed(chain.request().newBuilder()
+          .url("http://" + sameHost + ":" + differentPort + "/")
+          .build());
     };
     client = client.newBuilder()
         .addNetworkInterceptor(interceptor)
@@ -172,12 +163,10 @@ public final class InterceptorTest {
   @Test public void networkInterceptorsHaveConnectionAccess() throws Exception {
     server.enqueue(new MockResponse());
 
-    Interceptor interceptor = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        Connection connection = chain.connection();
-        assertNotNull(connection);
-        return chain.proceed(chain.request());
-      }
+    Interceptor interceptor = chain -> {
+      Connection connection = chain.connection();
+      assertNotNull(connection);
+      return chain.proceed(chain.request());
     };
     client = client.newBuilder()
         .addNetworkInterceptor(interceptor)
@@ -194,20 +183,18 @@ public final class InterceptorTest {
         .setBody(gzip("abcabcabc"))
         .addHeader("Content-Encoding: gzip"));
 
-    Interceptor interceptor = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        // The network request has everything: User-Agent, Host, Accept-Encoding.
-        Request networkRequest = chain.request();
-        assertNotNull(networkRequest.header("User-Agent"));
-        assertEquals(server.getHostName() + ":" + server.getPort(),
-            networkRequest.header("Host"));
-        assertNotNull(networkRequest.header("Accept-Encoding"));
+    Interceptor interceptor = chain -> {
+      // The network request has everything: User-Agent, Host, Accept-Encoding.
+      Request networkRequest = chain.request();
+      assertNotNull(networkRequest.header("User-Agent"));
+      assertEquals(server.getHostName() + ":" + server.getPort(),
+          networkRequest.header("Host"));
+      assertNotNull(networkRequest.header("Accept-Encoding"));
 
-        // The network response also has everything, including the raw gzipped content.
-        Response networkResponse = chain.proceed(networkRequest);
-        assertEquals("gzip", networkResponse.header("Content-Encoding"));
-        return networkResponse;
-      }
+      // The network response also has everything, including the raw gzipped content.
+      Response networkResponse = chain.proceed(networkRequest);
+      assertEquals("gzip", networkResponse.header("Content-Encoding"));
+      return networkResponse;
     };
     client = client.newBuilder()
         .addNetworkInterceptor(interceptor)
@@ -231,17 +218,15 @@ public final class InterceptorTest {
   @Test public void networkInterceptorsCanChangeRequestMethodFromGetToPost() throws Exception {
     server.enqueue(new MockResponse());
 
-    Interceptor interceptor = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        Request originalRequest = chain.request();
-        MediaType mediaType = MediaType.get("text/plain");
-        RequestBody body = RequestBody.create(mediaType, "abc");
-        return chain.proceed(originalRequest.newBuilder()
-            .method("POST", body)
-            .header("Content-Type", mediaType.toString())
-            .header("Content-Length", Long.toString(body.contentLength()))
-            .build());
-      }
+    Interceptor interceptor = chain -> {
+      Request originalRequest = chain.request();
+      MediaType mediaType = MediaType.get("text/plain");
+      RequestBody body = RequestBody.create(mediaType, "abc");
+      return chain.proceed(originalRequest.newBuilder()
+          .method("POST", body)
+          .header("Content-Type", mediaType.toString())
+          .header("Content-Length", Long.toString(body.contentLength()))
+          .build());
     };
     client = client.newBuilder()
         .addNetworkInterceptor(interceptor)
@@ -270,14 +255,12 @@ public final class InterceptorTest {
   private void rewriteRequestToServer(boolean network) throws Exception {
     server.enqueue(new MockResponse());
 
-    addInterceptor(network, new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        Request originalRequest = chain.request();
-        return chain.proceed(originalRequest.newBuilder()
-            .method("POST", uppercase(originalRequest.body()))
-            .addHeader("OkHttp-Intercepted", "yep")
-            .build());
-      }
+    addInterceptor(network, chain -> {
+      Request originalRequest = chain.request();
+      return chain.proceed(originalRequest.newBuilder()
+          .method("POST", uppercase(originalRequest.body()))
+          .addHeader("OkHttp-Intercepted", "yep")
+          .build());
     });
 
     Request request = new Request.Builder()
@@ -308,14 +291,12 @@ public final class InterceptorTest {
         .addHeader("Original-Header: foo")
         .setBody("abc"));
 
-    addInterceptor(network, new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        Response originalResponse = chain.proceed(chain.request());
-        return originalResponse.newBuilder()
-            .body(uppercase(originalResponse.body()))
-            .addHeader("OkHttp-Intercepted", "yep")
-            .build();
-      }
+    addInterceptor(network, chain -> {
+      Response originalResponse = chain.proceed(chain.request());
+      return originalResponse.newBuilder()
+          .body(uppercase(originalResponse.body()))
+          .addHeader("OkHttp-Intercepted", "yep")
+          .build();
     });
 
     Request request = new Request.Builder()
@@ -339,27 +320,23 @@ public final class InterceptorTest {
   private void multipleInterceptors(boolean network) throws Exception {
     server.enqueue(new MockResponse());
 
-    addInterceptor(network, new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        Request originalRequest = chain.request();
-        Response originalResponse = chain.proceed(originalRequest.newBuilder()
-            .addHeader("Request-Interceptor", "Android") // 1. Added first.
-            .build());
-        return originalResponse.newBuilder()
-            .addHeader("Response-Interceptor", "Donut") // 4. Added last.
-            .build();
-      }
+    addInterceptor(network, chain -> {
+      Request originalRequest = chain.request();
+      Response originalResponse = chain.proceed(originalRequest.newBuilder()
+          .addHeader("Request-Interceptor", "Android") // 1. Added first.
+          .build());
+      return originalResponse.newBuilder()
+          .addHeader("Response-Interceptor", "Donut") // 4. Added last.
+          .build();
     });
-    addInterceptor(network, new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        Request originalRequest = chain.request();
-        Response originalResponse = chain.proceed(originalRequest.newBuilder()
-            .addHeader("Request-Interceptor", "Bob") // 2. Added second.
-            .build());
-        return originalResponse.newBuilder()
-            .addHeader("Response-Interceptor", "Cupcake") // 3. Added third.
-            .build();
-      }
+    addInterceptor(network, chain -> {
+      Request originalRequest = chain.request();
+      Response originalResponse = chain.proceed(originalRequest.newBuilder()
+          .addHeader("Request-Interceptor", "Bob") // 2. Added second.
+          .build());
+      return originalResponse.newBuilder()
+          .addHeader("Response-Interceptor", "Cupcake") // 3. Added third.
+          .build();
     });
 
     Request request = new Request.Builder()
@@ -386,13 +363,11 @@ public final class InterceptorTest {
   private void asyncInterceptors(boolean network) throws Exception {
     server.enqueue(new MockResponse());
 
-    addInterceptor(network, new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        Response originalResponse = chain.proceed(chain.request());
-        return originalResponse.newBuilder()
-            .addHeader("OkHttp-Intercepted", "yep")
-            .build();
-      }
+    addInterceptor(network, chain -> {
+      Response originalResponse = chain.proceed(chain.request());
+      return originalResponse.newBuilder()
+          .addHeader("OkHttp-Intercepted", "yep")
+          .build();
     });
 
     Request request = new Request.Builder()
@@ -410,13 +385,12 @@ public final class InterceptorTest {
     server.enqueue(new MockResponse().setBody("b"));
 
     client = client.newBuilder()
-        .addInterceptor(new Interceptor() {
-          @Override public Response intercept(Chain chain) throws IOException {
-            Response response1 = chain.proceed(chain.request());
-            response1.body().close();
-            return chain.proceed(chain.request());
-          }
-        }).build();
+        .addInterceptor(chain -> {
+          Response response1 = chain.proceed(chain.request());
+          response1.body().close();
+          return chain.proceed(chain.request());
+        })
+        .build();
 
     Request request = new Request.Builder()
         .url(server.url("/"))
@@ -432,19 +406,18 @@ public final class InterceptorTest {
     server.enqueue(new MockResponse().setBody("b")); // Fetched directly.
 
     client = client.newBuilder()
-        .addInterceptor(new Interceptor() {
-          @Override public Response intercept(Chain chain) throws IOException {
-            if (chain.request().url().encodedPath().equals("/b")) {
-              Request requestA = new Request.Builder()
-                  .url(server.url("/a"))
-                  .build();
-              Response responseA = client.newCall(requestA).execute();
-              assertEquals("a", responseA.body().string());
-            }
-
-            return chain.proceed(chain.request());
+        .addInterceptor(chain -> {
+          if (chain.request().url().encodedPath().equals("/b")) {
+            Request requestA = new Request.Builder()
+                .url(server.url("/a"))
+                .build();
+            Response responseA = client.newCall(requestA).execute();
+            assertEquals("a", responseA.body().string());
           }
-        }).build();
+
+          return chain.proceed(chain.request());
+        })
+        .build();
 
     Request requestB = new Request.Builder()
         .url(server.url("/b"))
@@ -459,25 +432,24 @@ public final class InterceptorTest {
     server.enqueue(new MockResponse().setBody("b")); // Fetched directly.
 
     client = client.newBuilder()
-        .addInterceptor(new Interceptor() {
-          @Override public Response intercept(Chain chain) throws IOException {
-            if (chain.request().url().encodedPath().equals("/b")) {
-              Request requestA = new Request.Builder()
-                  .url(server.url("/a"))
-                  .build();
+        .addInterceptor(chain -> {
+          if (chain.request().url().encodedPath().equals("/b")) {
+            Request requestA = new Request.Builder()
+                .url(server.url("/a"))
+                .build();
 
-              try {
-                RecordingCallback callbackA = new RecordingCallback();
-                client.newCall(requestA).enqueue(callbackA);
-                callbackA.await(requestA.url()).assertBody("a");
-              } catch (Exception e) {
-                throw new RuntimeException(e);
-              }
+            try {
+              RecordingCallback callbackA = new RecordingCallback();
+              client.newCall(requestA).enqueue(callbackA);
+              callbackA.await(requestA.url()).assertBody("a");
+            } catch (Exception e) {
+              throw new RuntimeException(e);
             }
-
-            return chain.proceed(chain.request());
           }
-        }).build();
+
+          return chain.proceed(chain.request());
+        })
+        .build();
 
     Request requestB = new Request.Builder()
         .url(server.url("/b"))
@@ -500,11 +472,7 @@ public final class InterceptorTest {
    * with it.
    */
   private void interceptorThrowsRuntimeExceptionSynchronous(boolean network) throws Exception {
-    addInterceptor(network, new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        throw new RuntimeException("boom!");
-      }
-    });
+    addInterceptor(network, chain -> { throw new RuntimeException("boom!"); });
 
     Request request = new Request.Builder()
         .url(server.url("/"))
@@ -521,12 +489,12 @@ public final class InterceptorTest {
   @Test public void networkInterceptorModifiedRequestIsReturned() throws IOException {
     server.enqueue(new MockResponse());
 
-    Interceptor modifyHeaderInterceptor = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        return chain.proceed(chain.request().newBuilder()
-            .header("User-Agent", "intercepted request")
-            .build());
-      }
+    Interceptor modifyHeaderInterceptor = chain -> {
+      Request modifiedRequest = chain.request()
+          .newBuilder()
+          .header("User-Agent", "intercepted request")
+          .build();
+      return chain.proceed(modifiedRequest);
     };
 
     client = client.newBuilder()
@@ -557,11 +525,7 @@ public final class InterceptorTest {
    * exception goes to the uncaught exception handler.
    */
   private void interceptorThrowsRuntimeExceptionAsynchronous(boolean network) throws Exception {
-    addInterceptor(network, new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        throw new RuntimeException("boom!");
-      }
-    });
+    addInterceptor(network, chain -> { throw new RuntimeException("boom!"); });
 
     ExceptionCatchingExecutor executor = new ExceptionCatchingExecutor();
     client = client.newBuilder()
@@ -579,11 +543,9 @@ public final class InterceptorTest {
   @Test public void applicationInterceptorReturnsNull() throws Exception {
     server.enqueue(new MockResponse());
 
-    Interceptor interceptor = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        chain.proceed(chain.request());
-        return null;
-      }
+    Interceptor interceptor = chain -> {
+      chain.proceed(chain.request());
+      return null;
     };
     client = client.newBuilder()
         .addInterceptor(interceptor)
@@ -608,11 +570,9 @@ public final class InterceptorTest {
   @Test public void networkInterceptorReturnsNull() throws Exception {
     server.enqueue(new MockResponse());
 
-    Interceptor interceptor = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        chain.proceed(chain.request());
-        return null;
-      }
+    Interceptor interceptor = chain -> {
+      chain.proceed(chain.request());
+      return null;
     };
     client = client.newBuilder()
         .addNetworkInterceptor(interceptor)
@@ -639,12 +599,10 @@ public final class InterceptorTest {
         .setSocketPolicy(SocketPolicy.DISCONNECT_AT_END)
         .addHeader("Connection", "Close"));
 
-    Interceptor interceptor = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        Response response = chain.proceed(chain.request());
-        assertNotNull(chain.connection());
-        return response;
-      }
+    Interceptor interceptor = chain -> {
+      Response response = chain.proceed(chain.request());
+      assertNotNull(chain.connection());
+      return response;
     };
 
     client = client.newBuilder()
@@ -662,10 +620,11 @@ public final class InterceptorTest {
   @Test public void applicationInterceptorResponseMustHaveBody() throws Exception {
     server.enqueue(new MockResponse());
 
-    Interceptor interceptor = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        return chain.proceed(chain.request()).newBuilder().body(null).build();
-      }
+    Interceptor interceptor = chain -> {
+      Response response = chain.proceed(chain.request());
+      return response.newBuilder()
+          .body(null)
+          .build();
     };
     client = client.newBuilder()
         .addInterceptor(interceptor)
@@ -686,10 +645,11 @@ public final class InterceptorTest {
   @Test public void networkInterceptorResponseMustHaveBody() throws Exception {
     server.enqueue(new MockResponse());
 
-    Interceptor interceptor = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        return chain.proceed(chain.request()).newBuilder().body(null).build();
-      }
+    Interceptor interceptor = chain -> {
+      Response response = chain.proceed(chain.request());
+      return response.newBuilder()
+          .body(null)
+          .build();
     };
     client = client.newBuilder()
         .addNetworkInterceptor(interceptor)
@@ -708,33 +668,38 @@ public final class InterceptorTest {
   }
 
   @Test public void connectTimeout() throws Exception {
-    Interceptor interceptor1 = new Interceptor() {
-      @Override public Response intercept(Chain chainA) throws IOException {
-        assertEquals(5000, chainA.connectTimeoutMillis());
+    Interceptor interceptor1 = chainA -> {
+      assertEquals(5000, chainA.connectTimeoutMillis());
 
-        Chain chainB = chainA.withConnectTimeout(100, TimeUnit.MILLISECONDS);
-        assertEquals(100, chainB.connectTimeoutMillis());
+      Interceptor.Chain chainB = chainA.withConnectTimeout(100, TimeUnit.MILLISECONDS);
+      assertEquals(100, chainB.connectTimeoutMillis());
 
-        return chainB.proceed(chainA.request());
-      }
+      return chainB.proceed(chainA.request());
     };
 
-    Interceptor interceptor2 = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        assertEquals(100, chain.connectTimeoutMillis());
-        return chain.proceed(chain.request());
-      }
+    Interceptor interceptor2 = chain -> {
+      assertEquals(100, chain.connectTimeoutMillis());
+      return chain.proceed(chain.request());
     };
 
+    ServerSocket serverSocket = new ServerSocket(0, 1);
+    // Fill backlog queue with this request so subsequent requests will be blocked.
+    new Socket().connect(serverSocket.getLocalSocketAddress());
+
     client = client.newBuilder()
         .connectTimeout(5, TimeUnit.SECONDS)
         .addInterceptor(interceptor1)
         .addInterceptor(interceptor2)
         .build();
 
-    Request request1 = new Request.Builder()
-        .url("http://" + TestUtil.UNREACHABLE_ADDRESS)
-        .build();
+    Request request1 =
+        new Request.Builder()
+            .url(
+                "http://"
+                    + serverSocket.getInetAddress().getCanonicalHostName()
+                    + ":"
+                    + serverSocket.getLocalPort())
+            .build();
     Call call = client.newCall(request1);
 
     try {
@@ -742,25 +707,23 @@ public final class InterceptorTest {
       fail();
     } catch (SocketTimeoutException expected) {
     }
+
+    serverSocket.close();
   }
 
   @Test public void chainWithReadTimeout() throws Exception {
-    Interceptor interceptor1 = new Interceptor() {
-      @Override public Response intercept(Chain chainA) throws IOException {
-        assertEquals(5000, chainA.readTimeoutMillis());
+    Interceptor interceptor1 = chainA -> {
+      assertEquals(5000, chainA.readTimeoutMillis());
 
-        Chain chainB = chainA.withReadTimeout(100, TimeUnit.MILLISECONDS);
-        assertEquals(100, chainB.readTimeoutMillis());
+      Interceptor.Chain chainB = chainA.withReadTimeout(100, TimeUnit.MILLISECONDS);
+      assertEquals(100, chainB.readTimeoutMillis());
 
-        return chainB.proceed(chainA.request());
-      }
+      return chainB.proceed(chainA.request());
     };
 
-    Interceptor interceptor2 = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        assertEquals(100, chain.readTimeoutMillis());
-        return chain.proceed(chain.request());
-      }
+    Interceptor interceptor2 = chain -> {
+      assertEquals(100, chain.readTimeoutMillis());
+      return chain.proceed(chain.request());
     };
 
     client = client.newBuilder()
@@ -787,22 +750,18 @@ public final class InterceptorTest {
   }
 
   @Test public void chainWithWriteTimeout() throws Exception {
-    Interceptor interceptor1 = new Interceptor() {
-      @Override public Response intercept(Chain chainA) throws IOException {
-        assertEquals(5000, chainA.writeTimeoutMillis());
+    Interceptor interceptor1 = chainA -> {
+      assertEquals(5000, chainA.writeTimeoutMillis());
 
-        Chain chainB = chainA.withWriteTimeout(100, TimeUnit.MILLISECONDS);
-        assertEquals(100, chainB.writeTimeoutMillis());
+      Interceptor.Chain chainB = chainA.withWriteTimeout(100, TimeUnit.MILLISECONDS);
+      assertEquals(100, chainB.writeTimeoutMillis());
 
-        return chainB.proceed(chainA.request());
-      }
+      return chainB.proceed(chainA.request());
     };
 
-    Interceptor interceptor2 = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        assertEquals(100, chain.writeTimeoutMillis());
-        return chain.proceed(chain.request());
-      }
+    Interceptor interceptor2 = chain -> {
+      assertEquals(100, chain.writeTimeoutMillis());
+      return chain.proceed(chain.request());
     };
 
     client = client.newBuilder()
@@ -830,19 +789,17 @@ public final class InterceptorTest {
   }
 
   @Test public void chainCanCancelCall() throws Exception {
-    final AtomicReference<Call> callRef = new AtomicReference<>();
+    AtomicReference<Call> callRef = new AtomicReference<>();
 
-    Interceptor interceptor = new Interceptor() {
-      @Override public Response intercept(Chain chain) throws IOException {
-        Call call = chain.call();
-        callRef.set(call);
+    Interceptor interceptor = chain -> {
+      Call call = chain.call();
+      callRef.set(call);
 
-        assertFalse(call.isCanceled());
-        call.cancel();
-        assertTrue(call.isCanceled());
+      assertFalse(call.isCanceled());
+      call.cancel();
+      assertTrue(call.isCanceled());
 
-        return chain.proceed(chain.request());
-      }
+      return chain.proceed(chain.request());
     };
 
     client = client.newBuilder()
@@ -863,7 +820,7 @@ public final class InterceptorTest {
     assertSame(call, callRef.get());
   }
 
-  private RequestBody uppercase(final RequestBody original) {
+  private RequestBody uppercase(RequestBody original) {
     return new RequestBody() {
       @Override public MediaType contentType() {
         return original.contentType();
@@ -882,7 +839,7 @@ public final class InterceptorTest {
     };
   }
 
-  private Sink uppercase(final BufferedSink original) {
+  private Sink uppercase(BufferedSink original) {
     return new ForwardingSink(original) {
       @Override public void write(Buffer source, long byteCount) throws IOException {
         original.writeUtf8(source.readUtf8(byteCount).toUpperCase(Locale.US));
@@ -895,7 +852,7 @@ public final class InterceptorTest {
         Okio.buffer(uppercase(original.source())));
   }
 
-  private static Source uppercase(final Source original) {
+  private static Source uppercase(Source original) {
     return new ForwardingSource(original) {
       @Override public long read(Buffer sink, long byteCount) throws IOException {
         Buffer mixedCase = new Buffer();
@@ -929,17 +886,15 @@ public final class InterceptorTest {
     private final BlockingQueue<Exception> exceptions = new LinkedBlockingQueue<>();
 
     public ExceptionCatchingExecutor() {
-      super(1, 1, 0, TimeUnit.SECONDS, new SynchronousQueue<Runnable>());
+      super(1, 1, 0, TimeUnit.SECONDS, new SynchronousQueue<>());
     }
 
-    @Override public void execute(final Runnable runnable) {
-      super.execute(new Runnable() {
-        @Override public void run() {
-          try {
-            runnable.run();
-          } catch (Exception e) {
-            exceptions.add(e);
-          }
+    @Override public void execute(Runnable runnable) {
+      super.execute(() -> {
+        try {
+          runnable.run();
+        } catch (Exception e) {
+          exceptions.add(e);
         }
       });
     }
